@@ -2,15 +2,6 @@
 #define FILES_H
 
 #include "playlist.h"
-#include "file_types.h"
-
-enum file_type
-{
-	F_DIR,
-	F_SOUND,
-	F_PLAYLIST,
-	F_OTHER
-};
 
 struct file_list
 {
@@ -29,11 +20,12 @@ void resolve_path (char *buf, const int size, const char *file);
 char *ext_pos (const char *file);
 void file_list_free (struct file_list *list);
 struct file_list *file_list_new ();
-enum file_type file_type (char *file);
+enum file_type file_type (const char *file);
+int is_url (const char *str);
 char *read_line (FILE *file);
 void iconv_init ();
 void iconv_cleanup ();
-char *iconv_str (char *str);
+char *iconv_str (char *str, const int for_file_name);
 char *find_match_dir (char *dir);
 int file_exists (const char *file);
 time_t get_mtime (const char *file);
