@@ -2,7 +2,7 @@
 #define INTERFACE_H
 
 void init_interface (const int sock, const int logging, char **args,
-		const int arg_num);
+		const int arg_num, const int recursively);
 void interface_loop ();
 int server_connect ();
 void interface_end ();
@@ -20,4 +20,6 @@ void interface_cmdline_clear_plist (int server_sock);
 void interface_cmdline_append (int server_sock, char **args,
 		const int arg_num);
 void interface_cmdline_play_first (int server_sock);
+void interface_cmdline_file_info (const int server_sock);
+void interface_cmdline_playit (int server_sock, char **args, const int arg_num);
 #endif
