@@ -27,8 +27,8 @@ enum key_cmd
 	KEY_CMD_MIXER_INC_1,
 	KEY_CMD_MIXER_DEC_5,
 	KEY_CMD_MIXER_INC_5,
-	KEY_CMD_SEEK_FORWARD_1,
-	KEY_CMD_SEEK_BACKWARD_1,
+	KEY_CMD_SEEK_FORWARD,
+	KEY_CMD_SEEK_BACKWARD,
 	KEY_CMD_SEEK_FORWARD_5,
 	KEY_CMD_SEEK_BACKWARD_5,
 	KEY_CMD_HELP,
@@ -48,6 +48,29 @@ enum key_cmd
 	KEY_CMD_TOGGLE_SHUFFLE,
 	KEY_CMD_NEXT_SEARCH,
 	KEY_CMD_CANCEL,
+	KEY_CMD_GO_URL,
+	KEY_CMD_VOLUME_10,
+	KEY_CMD_VOLUME_20,
+	KEY_CMD_VOLUME_30,
+	KEY_CMD_VOLUME_40,
+	KEY_CMD_VOLUME_50,
+	KEY_CMD_VOLUME_60,
+	KEY_CMD_VOLUME_70,
+	KEY_CMD_VOLUME_80,
+	KEY_CMD_VOLUME_90,
+ 	KEY_CMD_FAST_DIR_1,
+ 	KEY_CMD_FAST_DIR_2,
+ 	KEY_CMD_FAST_DIR_3,
+ 	KEY_CMD_FAST_DIR_4,
+ 	KEY_CMD_FAST_DIR_5,
+ 	KEY_CMD_FAST_DIR_6,
+ 	KEY_CMD_FAST_DIR_7,
+ 	KEY_CMD_FAST_DIR_8,
+ 	KEY_CMD_FAST_DIR_9,
+ 	KEY_CMD_FAST_DIR_10,
+	KEY_CMD_TOGGLE_MIXER,
+	KEY_CMD_HISTORY_UP,
+	KEY_CMD_HISTORY_DOWN,
 	KEY_CMD_WRONG
 };
 
