@@ -41,6 +41,7 @@
 #endif
 
 void *xmalloc (const size_t size);
+void *xcalloc (size_t nmemb, size_t size);
 void *xrealloc (void *ptr, const size_t size);
 char *xstrdup (const char *s);
 
@@ -53,7 +54,6 @@ void error (const char *format, ...);
 #endif
 
 char *create_file_name (const char *file);
-int proper_sound_driver (const char *driver);
 int isdir (const char *file);
 
 #endif
