@@ -15,6 +15,7 @@
 
 #include <assert.h>
 #include <string.h>
+#include <strings.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -44,6 +45,7 @@ struct option
 	enum option_type type;
 	union option_value value;
 	int ignore_in_config;
+	int set_in_config;
 };
 
 static struct option options[OPTIONS_MAX];
@@ -60,6 +62,7 @@ static void option_add_int (const char *name, const int value)
 	options[options_num].type = OPTION_INT;
 	options[options_num].value.num = value;
 	options[options_num].ignore_in_config = 0;
+	options[options_num].set_in_config = 0;
 
 	options_num++;
 }
@@ -75,6 +78,7 @@ static void option_add_str (const char *name, const char *value)
 	options[options_num].type = OPTION_STR;
 	options[options_num].value.str = xstrdup (value);
 	options[options_num].ignore_in_config = 0;
+	options[options_num].set_in_config = 0;
 
 	options_num++;
 }
@@ -147,10 +151,12 @@ void options_init ()
 	option_add_str ("OSSDevice", "/dev/dsp");
 	option_add_str ("OSSMixerDevice", "/dev/mixer");
 	option_add_str ("OSSMixerChannel", "pcm");
-	option_add_str ("SoundDriver", "OSS");
+	option_add_str ("OSSMixerChannel2", "master");
+	option_add_str ("SoundDriver", "Jack, ALSA, OSS");
 	option_add_int ("ShowHiddenFiles", 1);
 	option_add_str ("AlsaDevice", "default");
 	option_add_str ("AlsaMixer", "PCM");
+	option_add_str ("AlsaMixer2", "Master");
 	option_add_int ("HideFileExtension", 0);
 	option_add_int ("ShowFormat", 1);
 	option_add_str ("ShowTime", "IfAvailable");
@@ -161,10 +167,31 @@ void options_init ()
 	option_add_int ("UseMmap", 1);
 	option_add_int ("Precache", 1);
 	option_add_str ("TagsIconv", NULL);
+	option_add_str ("FileNamesIconv", NULL);
 	option_add_int ("SavePlaylist", 1);
 	option_add_str ("Keymap", NULL);
 	option_add_int ("SyncPlaylist", 1);
+	option_add_int ("InputBuffer", 512);
+	option_add_int ("Prebuffering", 64);
+	option_add_str ("JackOutLeft", "alsa_pcm:playback_1");
+	option_add_str ("JackOutRight", "alsa_pcm:playback_2");
+	option_add_int ("ASCIILines", 0);
+	option_add_str ("FastDir1", NULL);
+	option_add_str ("FastDir2", NULL);
+	option_add_str ("FastDir3", NULL);
+	option_add_str ("FastDir4", NULL);
+	option_add_str ("FastDir5", NULL);
+	option_add_str ("FastDir6", NULL);
+	option_add_str ("FastDir7", NULL);
+	option_add_str ("FastDir8", NULL);
+	option_add_str ("FastDir9", NULL);
+	option_add_str ("FastDir10", NULL);
 	option_add_int ("Mp3IgnoreCRCErrors", 1);
+	option_add_int ("SeekTime", 1);
+	option_add_str ("ResampleMethod", "Linear");
+	option_add_int ("ForceSampleRate", 0);
+	option_add_str ("HTTPProxy", NULL);
+	option_add_int ("UseRealtimePriority", 0);
 }
 
 /* Return 1 if a parameter to an integer option is valid. */
@@ -191,6 +218,22 @@ int check_int_option (const char *name, const int val)
 		if (val < 128)
 			return 0;
 	}
+	else if (!strcasecmp(name, "InputBuffer")) {
+		if (val < 32)
+			return 0;
+	}
+	else if (!strcasecmp(name, "Prebuffering")) {
+		if (val < 0)
+			return 0;
+	}
+	else if (!strcasecmp(name, "SeekTime")) {
+		if (val < 1)
+			return 0;
+	}
+	else if (strcasecmp(name, "ForceSampleRate")) {
+		if (val < 0 || val > 500000)
+			return 0;
+	}
 	return 1;
 }
 
@@ -205,16 +248,13 @@ int check_str_option (const char *name, const char *val)
 		if (strcasecmp(val, "master") && strcasecmp(val, "pcm"))
 			return 0;
 	}
-	else if (!strcasecmp(name, "OutputDriver")) {
-		if (!proper_sound_driver(val))
-			return 0;
-	}
 	else if (!strcasecmp(name, "ShowTime")) {
 		if (strcasecmp(val, "yes") && strcasecmp(val, "no")
 				&& strcasecmp(val, "IfAvailable"))
 			return 0;
 	}
-	else if (!strcasecmp(name, "TagsIconv")) {
+	else if (!strcasecmp(name, "TagsIconv")
+			|| !strcasecmp(name, "FileNamesIconv")) {
 		char *colon;
 
 		/* simple check if val is FORMAT1:FORMAT2 */
@@ -222,6 +262,14 @@ int check_str_option (const char *name, const char *val)
 				|| *(colon + 1) == 0 || colon == val)
 			return 0;
 	}
+	else if (!strcmp(name, "ResampleMethod")) {
+		if (strcasecmp(val, "SincBestQuality")
+				&& strcasecmp(val, "SincMediumQuality")
+				&& strcasecmp(val, "SincFastest")
+				&& strcasecmp(val, "ZeroOrderHold")
+				&& strcasecmp(val, "Linear"))
+			return 0;
+	}
 	
 	return 1;
 }
@@ -231,12 +279,22 @@ static int set_option (const char *name, const char *value)
 {
 	int i = find_option (name, OPTION_ANY);
 
-	if (i == -1)
+	if (i == -1) {
+		fprintf (stderr, "Wrong option name: '%s'.", name);
 		return 0;
+	}
 
-	if (options[i].ignore_in_config)
+	if (options[i].ignore_in_config) 
 		return 1;
 
+	if (options[i].set_in_config) {
+		fprintf (stderr, "Tried to set an option that has been already "
+				"set in the config file ('%s').", name);
+		return 0;
+	}
+
+	options[i].set_in_config = 1;
+
 	if (options[i].type == OPTION_INT) {
 		int num;
 
@@ -265,6 +323,15 @@ static int set_option (const char *name, const char *value)
 	return 1;
 }
 
+/* Check if values of options make sense. This only check options that can't be
+ * check without parsing the whole file. */
+static void options_sanity_check ()
+{
+	if (options_get_int("Prebuffering") > options_get_int("InputBuffer"))
+		fatal ("Prebuffering is set to a value greater than "
+				"InputBuffer.");
+}
+
 /* Parse the configuration file. */
 void options_parse (const char *config_file)
 {
@@ -374,6 +441,8 @@ void options_parse (const char *config_file)
 		fatal ("Parse error at the end of the config file (need end of "
 				"line?).");
 
+	options_sanity_check ();
+
 	fclose (file);
 }
 
