@@ -30,11 +30,377 @@
 #include "log.h"
 #include "options.h"
 #include "files.h"
-#include "file_types.h"
 
 /* Initial size of the table */
 #define	INIT_SIZE	64
 
+enum rb_color { RED, BLACK };
+
+/* structure for Red-Black trees */
+struct rb_node
+{
+	struct rb_node *left;
+	struct rb_node *right;
+	struct rb_node *parent;
+	enum rb_color color;
+	int item_num;
+};
+
+/* item used as a null value */
+static struct rb_node rb_null = { NULL, NULL, NULL, BLACK, -1 };
+
+static void rb_left_rotate (struct rb_node **root, struct rb_node *x)
+{
+	struct rb_node *y = x->right;
+
+	assert (y != &rb_null);
+
+	x->right = y->left;
+	if (y->left != &rb_null)
+		y->left->parent = x;
+	y->parent = x->parent;
+
+	if (x->parent == &rb_null)
+		*root = y;
+	else {
+		if (x == x->parent->left)
+			x->parent->left = y;
+		else
+			x->parent->right = y;
+	}
+
+	y->left = x;
+	x->parent = y;
+}
+
+static void rb_right_rotate (struct rb_node **root, struct rb_node *x)
+{
+	struct rb_node *y = x->left;
+
+	assert (y != &rb_null);
+
+	x->left = y->right;
+	if (y->right != &rb_null)
+		y->right->parent = x;
+	y->parent = x->parent;
+
+	if (x->parent == &rb_null)
+		*root = y;
+	else {
+		if (x == x->parent->right)
+			x->parent->right = y;
+		else
+			x->parent->left = y;
+	}
+
+	y->right = x;
+	x->parent = y;
+}
+
+static void rb_insert_fixup (struct rb_node **root, struct rb_node *z)
+{
+	while (z->parent->color == RED)
+		if (z->parent == z->parent->parent->left) {
+			struct rb_node *y = z->parent->parent->right;
+			
+			if (y->color == RED) {
+				z->parent->color = BLACK;
+				y->color = BLACK;
+				z->parent->parent->color = RED;
+				z = z->parent->parent;
+			}
+			else {
+				if (z == z->parent->right) {
+					z = z->parent;
+					rb_left_rotate (root, z);
+				}
+
+				z->parent->color = BLACK;
+				z->parent->parent->color = RED;
+				rb_right_rotate (root, z->parent->parent);
+			}
+		}
+		else {
+			struct rb_node *y = z->parent->parent->left;
+			
+			if (y->color == RED) {
+				z->parent->color = BLACK;
+				y->color = BLACK;
+				z->parent->parent->color = RED;
+				z = z->parent->parent;
+			}
+			else {
+				if (z == z->parent->left) {
+					z = z->parent;
+					rb_right_rotate (root, z);
+				}
+				
+				z->parent->color = BLACK;
+				z->parent->parent->color = RED;
+				rb_left_rotate (root, z->parent->parent);
+			}
+		}
+	
+	(*root)->color = BLACK;
+}
+
+static void rb_delete_fixup (struct rb_node **root, struct rb_node *x,
+		struct rb_node *parent)
+{
+	struct rb_node *w;
+	
+	while (x != *root && x->color == BLACK) {
+		if (x == parent->left) {
+			w = parent->right;
+
+			if (w->color == RED) {
+				w->color = BLACK;
+				parent->color = RED;
+				rb_left_rotate (root, parent);
+				w = parent->right;
+			}
+
+			if (w->left->color == BLACK
+					&& w->right->color == BLACK) {
+				w->color = RED;
+				x = parent;
+				parent = x->parent;
+			}
+			else {
+				if (w->right->color == BLACK) {
+					w->left->color = BLACK;
+					w->color = RED;
+					rb_right_rotate (root, w);
+					w = parent->right;
+				}
+				
+				w->color = parent->color;
+				parent->color = BLACK;
+				w->right->color = BLACK;
+				rb_left_rotate (root, parent);
+				x = *root;
+			}
+		}
+		else {
+			w = parent->left;
+
+			if (w->color == RED) {
+				w->color = BLACK;
+				parent->color = RED;
+				rb_right_rotate (root, parent);
+				w = parent->left;
+			}
+
+			if (w->right->color == BLACK
+					&& w->left->color == BLACK) {
+				w->color = RED;
+				x = parent;
+				parent = x->parent;
+			}
+			else {
+				if (w->left->color == BLACK) {
+					w->right->color = BLACK;
+					w->color = RED;
+					rb_left_rotate (root, w);
+					w = parent->left;
+				}
+
+				w->color = parent->color;
+				parent->color = BLACK;
+				w->left->color = BLACK;
+				rb_right_rotate (root, parent);
+				x = *root;
+			}
+		}
+
+	}
+
+	x->color = BLACK;
+}
+
+/* Insert the playlist position into the reb-black tree. */
+static void rb_insert (struct rb_node **root, const struct plist *plist,
+		const int num)
+{
+	struct rb_node *z;
+	struct rb_node *y = &rb_null;
+	struct rb_node *x = *root;
+
+	assert (root != NULL);
+	assert (num >= 0);
+	assert (plist->items[num].file != NULL);
+
+	z = (struct rb_node *)xmalloc (sizeof(struct rb_node));
+
+	z->item_num = num;
+	
+	while (x != &rb_null) {
+		y = x;
+		if (strcoll(plist->items[z->item_num].file,
+					plist->items[x->item_num].file) < 0)
+			x = x->left;
+		else
+			x = x->right;
+	}
+
+	z->parent = y;
+	if (y == &rb_null)
+		*root = z;
+	else {
+		if (strcoll(plist->items[z->item_num].file,
+					plist->items[y->item_num].file)	< 0)
+			y->left = z;
+		else
+			y->right = z;
+	}
+	
+	z->left = &rb_null;
+	z->right = &rb_null;
+	z->color = RED;
+
+	rb_insert_fixup (root, z);
+}
+
+static void rb_destroy (struct rb_node *root)
+{
+	assert (root != NULL);
+
+	if (root != &rb_null) {	
+		rb_destroy (root->left);
+		rb_destroy (root->right);
+		free (root);
+	}
+}
+
+static struct rb_node *rb_search_internal (struct rb_node *root,
+		const struct plist *plist, const char *file)
+{
+	struct rb_node *x = root;
+
+	while (x != &rb_null) {
+		int cmp = strcoll (file, plist->items[x->item_num].file);
+		
+		if (cmp < 0)
+			x = x->left;
+		else if (cmp > 0)
+			x = x->right;
+		else
+			return x;
+	}
+
+	return NULL;
+}
+
+/* Search in the red-black tree for the item with file name file.
+ * Return the item position on the playlist or -1 if not found. */
+static int rb_search (struct rb_node *root, const struct plist *plist,
+		const char *file)
+{
+	const struct rb_node *x = rb_search_internal (root, plist, file);
+	
+	return x ? x->item_num : -1;
+}
+
+static struct rb_node *rb_min (struct rb_node *root)
+{
+	struct rb_node *x = root;
+
+	if (root == &rb_null)
+		return &rb_null;
+
+	while (x->left != &rb_null)
+		x = x->left;
+
+	return x;
+}
+
+#if 0
+static struct rb_node *rb_max (struct rb_node *root)
+{
+	struct rb_node *x = root;
+
+	if (root == &rb_null)
+		return NULL;
+
+	while (x->right != &rb_null)
+		x = x->right;
+
+	return x;
+}
+#endif
+
+static struct rb_node *rb_next (struct rb_node *x)
+{
+	struct rb_node *y;
+	
+	if (x->right != &rb_null)
+		return rb_min (x->right);
+	
+	y = x->parent;
+	while (y != &rb_null && x == y->right) {
+		x = y;
+		y = y->parent;
+	}
+
+	return y;
+}
+
+static void rb_delete (struct rb_node **root, const struct plist *plist,
+		const char *file)
+{
+	struct rb_node *z = rb_search_internal (*root, plist, file);
+
+	if (z) {
+		struct rb_node *x, *y, *parent;
+		
+		if (z->left == &rb_null || z->right == &rb_null) 
+			y = z;
+		else
+			y = rb_next (z);
+
+		if (y->left != &rb_null)
+			x = y->left;
+		else
+			x = y->right;
+
+		parent = y->parent;
+		if (x != &rb_null)
+			x->parent = parent;
+		
+		if (y->parent == &rb_null)
+			*root = x;
+		else {
+			if (y == y->parent->left)
+				y->parent->left = x;
+			else
+				y->parent->right = x;
+		}
+
+		if (y != z)
+			z->item_num = y->item_num;
+
+		if (y->color == BLACK)
+			rb_delete_fixup (root, x, parent);
+
+		free (y);
+	}
+}
+
+/* Update playlist position for item with the given file name. Return 0 if
+ * there is no such item. */
+static int rb_update_position (struct rb_node *root, const struct plist *plist,
+		const char *file, const int new_pos)
+{
+	struct rb_node *x = rb_search_internal (root, plist, file);
+
+	if (x) {
+		x->item_num = new_pos;
+		return 1;
+	}
+
+	return 0;
+}
+
 void tags_free (struct file_tags *tags)
 {
 	assert (tags != NULL);
@@ -49,6 +415,44 @@ void tags_free (struct file_tags *tags)
 	free (tags);
 }
 
+void tags_clear (struct file_tags *tags)
+{
+	assert (tags != NULL);
+
+	if (tags->title)
+		free (tags->title);
+	if (tags->artist)
+		free (tags->artist);
+	if (tags->album)
+		free (tags->album);
+
+	tags->title = NULL;
+	tags->artist = NULL;
+	tags->album = NULL;
+	tags->track = -1;
+	tags->time = -1;
+}
+
+/* Copy the tags data from src to dst freeing old fields if necessary. */
+void tags_copy (struct file_tags *dst, const struct file_tags *src)
+{
+	if (dst->title)
+		free (dst->title);
+	dst->title = xstrdup (src->title);
+
+	if (dst->artist)
+		free (dst->artist);
+	dst->artist = xstrdup (src->artist);
+
+	if (dst->album)
+		free (dst->album);
+	dst->album = xstrdup (src->album);
+	
+	dst->track = src->track;
+	dst->time = src->time;
+	dst->filled = src->filled;
+}
+
 struct file_tags *tags_new ()
 {
 	struct file_tags *tags;
@@ -64,19 +468,15 @@ struct file_tags *tags_new ()
 	return tags;
 }
 
-static struct file_tags *tags_dup (const struct file_tags *tags)
+struct file_tags *tags_dup (const struct file_tags *tags)
 {
-
 	struct file_tags *dtags;
 
-	dtags = (struct file_tags *)xmalloc (sizeof(struct file_tags));
-	dtags->title = xstrdup (tags->title);
-	dtags->artist = xstrdup (tags->artist);
-	dtags->album = xstrdup (tags->album);
-	dtags->track = tags->track;
-	dtags->time = tags->time;
-	dtags->filled = tags->filled;
-	
+	assert (tags != NULL);
+
+	dtags = tags_new();
+	tags_copy (dtags, tags);
+
 	return dtags;
 }
 
@@ -116,21 +516,9 @@ void plist_init (struct plist *plist)
 	plist->items = (struct plist_item *)xmalloc (sizeof(struct plist_item)
 			* INIT_SIZE);
 	plist->serial = -1;
-}
-
-static int str_hash (const char *file)
-{
-	long h = 0;
-	int i = 0;
-
-	assert (file != NULL);
-
-	while (file[i]) {
-		h ^= i % 2 ? file[i] : file[i] << 8;
-		i++;
-	}
-
-	return h;
+	plist->search_tree = &rb_null;
+	plist->total_time = 0;
+	plist->time_for_all_files = 0;
 }
 
 /* Create a new playlit item with empty fields. */
@@ -140,7 +528,7 @@ struct plist_item *plist_new_item ()
 
 	item = (struct plist_item *)xmalloc (sizeof(struct plist_item));
 	item->file = NULL;
-	item->file_hash = -1;
+	item->type = F_OTHER;
 	item->deleted = 0;
 	item->title = NULL;
 	item->title_file = NULL;
@@ -164,8 +552,7 @@ int plist_add (struct plist *plist, const char *file_name)
 	}
 
 	plist->items[plist->num].file = xstrdup (file_name);
-	plist->items[plist->num].file_hash = file_name
-		? str_hash (file_name) : -1;
+	plist->items[plist->num].type = F_OTHER;
 	plist->items[plist->num].deleted = 0;
 	plist->items[plist->num].title = NULL;
 	plist->items[plist->num].title_file = NULL;
@@ -173,6 +560,13 @@ int plist_add (struct plist *plist, const char *file_name)
 	plist->items[plist->num].tags = NULL;
 	plist->items[plist->num].mtime = (file_name ? get_mtime(file_name)
 			: (time_t)-1);
+
+	if (file_name) {
+		if (!rb_update_position(plist->search_tree, plist, file_name,
+					plist->num))
+			rb_insert (&plist->search_tree, plist, plist->num);
+	}
+	
 	plist->num++;
 	plist->not_deleted++;
 
@@ -182,9 +576,10 @@ int plist_add (struct plist *plist, const char *file_name)
 /* Copy all fields of item src to dst. */
 void plist_item_copy (struct plist_item *dst, const struct plist_item *src)
 {
+	if (dst->file)
+		free (dst->file);
 	dst->file = xstrdup (src->file);
-	dst->file_hash = src->file_hash != -1
-		? src->file_hash : (src->file ? str_hash(src->file) : -1);
+	dst->type = src->type;
 	dst->title_file = xstrdup (src->title_file);
 	dst->title_tags = xstrdup (src->title_tags);
 	dst->mtime = src->mtime;
@@ -274,7 +669,6 @@ void plist_free_item_fields (struct plist_item *item)
 		item->tags = NULL;
 	}
 
-	item->file_hash = -1;
 	item->title = NULL;
 }
 
@@ -294,6 +688,10 @@ void plist_clear (struct plist *plist)
 	plist->allocated = INIT_SIZE;
 	plist->num = 0;
 	plist->not_deleted = 0;
+	rb_destroy (plist->search_tree);
+	plist->search_tree = &rb_null;
+	plist->total_time = 0;
+	plist->time_for_all_files = 0;
 }
 
 /* Destroy the list freeing memory, the list can't be used after that. */
@@ -307,41 +705,50 @@ void plist_free (struct plist *plist)
 	plist->items = NULL;
 }
 
-static int qsort_func_fname (const void *a, const void *b)
-{
-	struct plist_item *ap = (struct plist_item *)a;
-	struct plist_item *bp = (struct plist_item *)b;
-
-	if (ap->deleted || bp->deleted)
-		return ap->deleted - bp->deleted;
-	
-	return strcmp (ap->file, bp->file);
-}
-
 /* Sort the playlist by file names. */
 void plist_sort_fname (struct plist *plist)
 {
-	qsort (plist->items, plist->num, sizeof(struct plist_item),
-			qsort_func_fname);
+	struct plist_item *sorted;
+	struct rb_node *x;
+	int n;
+
+	if (plist_count(plist) == 0)
+		return;
+
+	sorted = (struct plist_item *)xmalloc (plist_count(plist) *
+			sizeof(struct plist_item));
+
+	x = rb_min (plist->search_tree);
+	while (plist_deleted(plist, x->item_num))
+		x = rb_next (x);
+	
+	sorted[0] = plist->items[x->item_num];
+	x->item_num = 0;
+	
+	n = 1;
+	while ((x = rb_next(x)) != &rb_null)
+		if (!plist_deleted(plist, x->item_num)) {
+			sorted[n] = plist->items[x->item_num];
+			x->item_num = n++;
+		}
+
+	plist->num = n;
+	plist->not_deleted = n;
+	
+	memcpy (plist->items, sorted, sizeof(struct plist_item) * n);
+	free (sorted);
 }
 
 /* Find an item on the list items. Return the index or -1 if not found. */
 int plist_find_fname (struct plist *plist, const char *file)
 {
 	int i;
-	int hash = str_hash (file);
-
+	
 	assert (plist != NULL);
 
-	for (i = 0; i < plist->num; i++) {
-		if (!plist_deleted(plist, i) && plist->items[i].file
-				&& plist->items[i].file_hash == hash
-				&& !strcmp(plist->items[i].file, file)) {
-			return i;
-		}
-	}
+	i = rb_search (plist->search_tree, plist, file);
 
-	return -1;
+	return i != -1 && !plist_deleted(plist, i) ? i : -1;
 }
 
 /* Find an item on the list, also find deleted items. If there are more than one
@@ -351,13 +758,11 @@ int plist_find_del_fname (struct plist *plist, const char *file)
 {
 	int i;
 	int item = -1;
-	int hash = str_hash (file);
 
 	assert (plist != NULL);
 
 	for (i = 0; i < plist->num; i++) {
 		if (plist->items[i].file
-				&& plist->items[i].file_hash == hash
 				&& !strcmp(plist->items[i].file, file)) {
 			if (item == -1 || plist_deleted(plist, item))
 				item = i;
@@ -516,7 +921,7 @@ char *build_title (const struct file_tags *tags)
 /* Copy the item to the playlist. Return the index of the added item. */
 int plist_add_from_item (struct plist *plist, const struct plist_item *item)
 {
-	int pos = plist_add (plist, NULL);
+	int pos = plist_add (plist, item->file);
 
 	plist_item_copy (&plist->items[pos], item);
 
@@ -533,7 +938,7 @@ void plist_delete (struct plist *plist, const int num)
 
 		/* Free every field except the file, it is needed in deleted
 		 * items. */
-		char *file = plist_get_file (plist, num);
+		char *file = plist->items[num].file;
 
 		plist->items[num].file = NULL;
 		plist_free_item_fields (&plist->items[num]);
@@ -580,11 +985,15 @@ void plist_set_file (struct plist *plist, const int num, const char *file)
 	assert (num >=0 && num < plist->num);
 	assert (file != NULL);
 
-	if (plist->items[num].file)
+	if (plist->items[num].file) {
+		rb_delete (&plist->search_tree, plist, file);
 		free (plist->items[num].file);
+		plist->items[num].type = F_OTHER;
+	}
+	
 	plist->items[num].file = xstrdup (file);
-	plist->items[num].file_hash = str_hash (file);
 	plist->items[num].mtime = get_mtime (file);
+	rb_insert (&plist->search_tree, plist, num);
 }
 
 /* Add the content of playlist b to a by copying items. */
@@ -598,7 +1007,8 @@ void plist_cat (struct plist *a, struct plist *b)
 	for (i = 0; i < b->num; i++) {
 		assert (b->items[i].file != NULL);
 		
-		if (plist_find_fname(a, b->items[i].file) == -1)
+		if (!plist_deleted(b, i)
+				&& plist_find_fname(a, b->items[i].file) == -1)
 			plist_add_from_item (a, &b->items[i]);
 	}
 }
@@ -685,15 +1095,15 @@ int get_item_time (const struct plist *plist, const int i)
 	return -1;
 }
 
-/* Return the total time of all files on the playlist that has the time tag.
- * If the time information is missing for any file, all_files is set to 0,
- * otherwise 1. */
-int plist_total_time (const struct plist *plist, int *all_files)
+/* Count total time of items on the playlist. */
+void plist_count_total_time (struct plist *plist)
 {
 	int i;
-	int total_time = 0;
 
-	*all_files = 1;
+	assert (plist != NULL);
+
+	plist->time_for_all_files = 1;
+	plist->total_time = 0;
 
 	for (i = 0; i < plist->num; i++)
 		if (!plist_deleted(plist, i)) {
@@ -701,12 +1111,23 @@ int plist_total_time (const struct plist *plist, int *all_files)
 					|| !(plist->items[i].tags->filled
 						& TAGS_TIME)
 					|| plist->items[i].tags->time == -1)
-				*all_files = 0;
+				plist->time_for_all_files = 0;
 			else
-				total_time += plist->items[i].tags->time;
+				plist->total_time +=
+					plist->items[i].tags->time;
 		}
+}
 
-	return total_time;
+/* Return the total time of all files on the playlist that has the time tag.
+ * If the time information is missing for any file, all_files is set to 0,
+ * otherwise 1.
+ * Returned value is that counted by plist_count_time(), so may be not
+ * up-to-date. */
+int plist_total_time (const struct plist *plist, int *all_files)
+{
+	*all_files = plist->time_for_all_files;
+	
+	return plist->total_time;
 }
 
 /* Swap two items on the playlist. */
@@ -733,6 +1154,12 @@ void plist_shuffle (struct plist *plist)
 	for (i = 0; i < plist->num; i++)
 		plist_swap (plist, i,
 				(rand()/(float)RAND_MAX) * (plist->num - 1));
+
+	rb_destroy (plist->search_tree);
+	plist->search_tree = &rb_null;
+	
+	for (i = 0; i < plist->num; i++)
+		rb_insert (&plist->search_tree, plist, i);
 }
 
 /* Swap the first item on the playlist with the item with file fname. */
@@ -745,8 +1172,13 @@ void plist_swap_first_fname (struct plist *plist, const char *fname)
 	
 	i = plist_find_fname (plist, fname);
 
-	if (i != -1)
+	if (i != -1 && i != 0) {
+		rb_delete (&plist->search_tree, plist, fname);
+		rb_delete (&plist->search_tree, plist, plist->items[0].file);
 		plist_swap (plist, 0, i);
+		rb_insert (&plist->search_tree, plist, 0);
+		rb_insert (&plist->search_tree, plist, i);
+	}
 }
 
 void plist_set_serial (struct plist *plist, const int serial)
@@ -772,3 +1204,27 @@ int plist_last (struct plist *plist)
 
 	return i;
 }
+
+enum file_type plist_file_type (struct plist *plist, const int num)
+{
+	assert (plist != NULL);
+	assert (num < plist->num);
+
+	if (plist->items[num].type != F_OTHER)
+		return plist->items[num].type;
+
+	return (plist->items[num].type = file_type(plist->items[num].file));
+}
+
+/* Remove items from playlist a that are also presend on playlist b. */
+void plist_remove_common_items (struct plist *a, struct plist *b)
+{
+	int i;
+
+	assert (a != NULL);
+	assert (b != NULL);
+	
+	for (i = 0; i < a->num; i++)
+		if (plist_find_fname(b, a->items[i].file) != -1)
+			plist_delete (a, i);
+}
