@@ -95,6 +95,7 @@ int send_int (int sock, int i)
 	return res == sizeof(int) ? 1 : 0;
 }
 
+#if 0
 /* Get a long value from the socket, return == 0 on error. */
 static int get_long (int sock, long *i)
 {
@@ -107,7 +108,6 @@ static int get_long (int sock, long *i)
 	return res == sizeof(long) ? 1 : 0;
 }
 
-#if 0
 /* Send a long value to the socket, return == 0 on error */
 static int send_long (int sock, long i)
 {
@@ -233,13 +233,14 @@ static char *add_buf_int (char *buf, size_t *len, size_t *allocated,
 	return buf;
 }
 
+#if 0
 /* Add a long to the dynamicaly allocated buffer which has aready len bytes
  * data and is allocated big. Returns the pointer to the buffer which may be not
  * the same as buf. */
 static char *add_buf_long (char *buf, size_t *len, size_t *allocated,
 		const int n)
 {
-	if (*allocated - *len < sizeof(n)) {
+	if (*allocated - *len < (int)sizeof(n)) {
 		*allocated *= 2;
 		buf = xrealloc (buf, *allocated);
 	}
@@ -249,6 +250,7 @@ static char *add_buf_long (char *buf, size_t *len, size_t *allocated,
 
 	return buf;
 }
+#endif
 
 /* Add a time_t to the dynamicaly allocated buffer which has aready len bytes
  * data and is allocated big. Returns the pointer to the buffer which may be not
@@ -256,7 +258,7 @@ static char *add_buf_long (char *buf, size_t *len, size_t *allocated,
 static char *add_buf_time (char *buf, size_t *len, size_t *allocated,
 		const time_t t)
 {
-	if (*allocated - *len < sizeof(t)) {
+	if (*allocated - *len < (int)sizeof(t)) {
 		*allocated *= 2;
 		buf = xrealloc (buf, *allocated);
 	}
@@ -267,34 +269,45 @@ static char *add_buf_time (char *buf, size_t *len, size_t *allocated,
 	return buf;
 }
 
-/* Make a apcket from the item fields suitable to send() as. The returned
- * memory is malloc()ed and the size of the packed is put into len. */
-static char *make_item_packet (const struct plist_item *item, size_t *len)
+/* Add data to the dynamicaly allocated buffer which has aready len bytes
+ * data and is allocated big. Returns the pointer to the buffer which may be not
+ * the same as buf. */
+static char *add_buf_data (char *buf, size_t *len, size_t *allocated,
+		const char *data, const size_t data_len)
+{
+	if (*allocated - *len < data_len) {
+		*allocated *= 2;
+		buf = xrealloc (buf, *allocated);
+	}
+
+	memcpy (buf + *len, data, data_len);
+	*len += data_len;
+
+	return buf;
+}
+
+/* Make a packet from the tags fields suitable to send() as. The returned
+ * memory is malloc()ed and the size of the packet is put into len.
+ * If tags == NULL, make a packet with empty tags. */
+static char *make_tags_packet (const struct file_tags *tags, size_t *len)
 {
 	char *buf;
-	size_t allocated;
-
-	allocated = 2048;
-	buf = xmalloc (allocated);
+	size_t allocated = 2048;
+	
+	buf = (char *)xmalloc (allocated);
 	*len = 0;
 
-	buf = add_buf_str (buf, len, &allocated, item->file);
-	buf = add_buf_long (buf, len, &allocated, item->file_hash);
-	buf = add_buf_str (buf, len, &allocated,
-			item->title_tags ? item->title_tags : "");
-	
-	if (item->tags) {
+	if (tags) {
 		buf = add_buf_str (buf, len, &allocated,
-				item->tags->title ? item->tags->title : "");
+				tags->title ? tags->title : "");
 		buf = add_buf_str (buf, len, &allocated,
-				item->tags->artist ? item->tags->artist : "");
+				tags->artist ? tags->artist : "");
 		buf = add_buf_str (buf, len, &allocated,
-				item->tags->album ? item->tags->album : "");
-		buf = add_buf_int (buf, len, &allocated, item->tags->track);
+				tags->album ? tags->album : "");
+		buf = add_buf_int (buf, len, &allocated, tags->track);
 		
 		buf = add_buf_int (buf, len, &allocated,
-				item->tags->filled & TAGS_TIME
-				? item->tags->time : -1);
+				tags->filled & TAGS_TIME ? tags->time : -1);
 	}
 	else {
 		
@@ -306,18 +319,60 @@ static char *make_item_packet (const struct plist_item *item, size_t *len)
 		buf = add_buf_int (buf, len, &allocated, -1); /* time */
 	}
 
+	return buf;
+}
+
+/* Make a packet from the item fields suitable to send() as. The returned
+ * memory is malloc()ed and the size of the packet is put into len. */
+static char *make_item_packet (const struct plist_item *item, size_t *len)
+{
+	char *buf;
+	size_t allocated;
+	char *tags_packet;
+	size_t tags_packet_len;
+
+	allocated = 2048;
+	buf = (char *)xmalloc (allocated);
+	*len = 0;
+
+	buf = add_buf_str (buf, len, &allocated, item->file);
+	buf = add_buf_str (buf, len, &allocated,
+			item->title_tags ? item->title_tags : "");
+
+	tags_packet = make_tags_packet (item->tags, &tags_packet_len);
+	buf = add_buf_data (buf, len, &allocated, tags_packet, tags_packet_len);
+	free (tags_packet);
+	
 	buf = add_buf_time (buf, len, &allocated, item->mtime);
 
 	return buf;
 }
 
+/* Send data to the socket. Return 0 on error. */
+static int send_all (int sock, const char *buf, const size_t size)
+{
+	ssize_t sent;
+	size_t send_pos = 0;
+	
+	while (send_pos < size) {
+		sent = send (sock, buf + send_pos, size - send_pos, 0);
+		if (sent < 0) {
+			logit ("Error while sending data: %s", strerror(errno));
+			return 0;
+		}
+		send_pos += sent;
+	}
+
+	return 1;
+}
+
 /* Send a playlist item to the socket. If item == NULL, send empty item mark
  * (end of playlist). Return 0 on error. */
 int send_item (int sock, const struct plist_item *item)
 {
 	char *buf;
-	ssize_t sent;
-	size_t send_pos = 0, pkt_size;
+	size_t pkt_size;
+	int res = 1;
 	
 	if (!item) {
 		if (!send_str(sock, "")) {
@@ -328,20 +383,86 @@ int send_item (int sock, const struct plist_item *item)
 	}
 
 	buf = make_item_packet (item, &pkt_size);
-	
-	while (send_pos < pkt_size) {
-		sent = send (sock, buf + send_pos, pkt_size - send_pos, 0);
-		if (sent < 0) {
-			logit ("Error while sendint item: %s", strerror(errno));
-			free (buf);
-			return 0;
-		}
-		send_pos += sent;
+	if (!send_all(sock, buf, pkt_size)) {
+		logit ("Error when sending item");
+		res = 0;
 	}
 		
 	free (buf);
+	return res;
+}
 
-	return 1;
+struct file_tags *recv_tags (int sock)
+{
+	struct file_tags *tags = tags_new ();
+
+	if (!(tags->title = get_str(sock))) {
+		logit ("Error while receiving titile");
+		tags_free (tags);
+		return NULL;
+	}
+	
+	if (!(tags->artist = get_str(sock))) {
+		logit ("Error while receiving artist");
+		tags_free (tags);
+		return NULL;
+	}
+	
+	if (!(tags->album = get_str(sock))) {
+		logit ("Error while receiving album");
+		tags_free (tags);
+		return NULL;
+	}
+		
+	if (!get_int(sock, &tags->track)) {
+		logit ("Error while receiving ");
+		tags_free (tags);
+		return NULL;
+	}
+	
+	if (!get_int(sock, &tags->time)) {
+		logit ("Error while receiving time");
+		tags_free (tags);
+		return NULL;
+	}
+
+	if (tags->time != -1)
+		tags->filled |= TAGS_TIME;
+
+	/* Set NULL instead of empty tags. */
+	if (!tags->title[0]) {
+		free (tags->title);
+		tags->title = NULL;
+	}
+	if (!tags->artist[0]) {
+		free (tags->artist);
+		tags->artist = NULL;
+	}
+	if (!tags->album[0]) {
+		free (tags->album);
+		tags->album = NULL;
+	}
+
+	if (tags->title || tags->artist || tags->album || tags->track != -1)
+		tags->filled |= TAGS_COMMENTS;
+
+	return tags;
+}
+
+/* Send tags. If tags == NULL, send empty tags. REturn 0 on error. */
+int send_tags (int sock, const struct file_tags *tags)
+{
+	char *packet;
+	size_t packet_size;
+	int res = 1;
+
+	packet = make_tags_packet (tags, &packet_size);
+	
+	if (!send_all(sock, packet, packet_size))
+		res = 0;
+	
+	free (packet);
+	return res;
 }
 
 /* Get a playlist item from the server. If empty item->file is an empty string,
@@ -359,15 +480,6 @@ struct plist_item *recv_item (int sock)
 	}
 
 	if (item->file[0]) {
-		char *title, *artist, *album;
-		int track, time;
-		
-		if (!(get_long(sock, &item->file_hash))) {
-			logit ("Error while receiving file hash");
-			free (item->file);
-			free (item);
-		}
-
 		if (!(item->title_tags = get_str(sock))) {
 			logit ("Error while receiving tags title");
 			free (item->file);
@@ -378,96 +490,24 @@ struct plist_item *recv_item (int sock)
 			free (item->title_tags);
 			item->title_tags = NULL;
 		}
-		
-		if (!(title = get_str(sock))) {
-			logit ("Error while receiving titile");
+
+		if (!(item->tags = recv_tags(sock))) {
+			logit ("Error while receiving tags");
+			free (item->file);
 			if (item->title_tags)
 				free (item->title_tags);
 			free (item);
-			return NULL;
-		}
-		
-		if (!(artist = get_str(sock))) {
-			logit ("Error while receiving artist");
-			if (item->title_tags)
-				free (item->title_tags);
-			free (title);
-			free (item);
-			return NULL;
-		}
-		
-		if (!(album = get_str(sock))) {
-			logit ("Error while receiving album");
-			if (item->title_tags)
-				free (item->title_tags);
-			free (title);
-			free (artist);
-			free (item);
-			return NULL;
-		}
-			
-		if (!get_int(sock, &track)) {
-			logit ("Error while receiving ");
-			if (item->title_tags)
-				free (item->title_tags);
-			free (title);
-			free (artist);
-			free (item);
-			return NULL;
-		}
-		
-		if (!get_int(sock, &time)) {
-			logit ("Error while receiving time");
-			if (item->title_tags)
-				free (item->title_tags);
-			free (title);
-			free (artist);
-			free (item);
-			return NULL;
 		}
 		
 		if (!get_time(sock, &item->mtime)) {
 			logit ("Error while receiving mtime");
 			if (item->title_tags)
 				free (item->title_tags);
-			free (title);
-			free (artist);
-			free (item);
+			free (item->file);
+			tags_free (item->tags);
 			return NULL;
 		}
 
-		/* Set NULL instead of empty tags. */
-		if (!title[0]) {
-			free (title);
-			title = NULL;
-		}
-		if (!artist[0]) {
-			free (artist);
-			artist = NULL;
-		}
-		if (!album[0]) {
-			free (album);
-			album = NULL;
-		}
-
-		if (title || artist || album || track != -1 || time != -1) {
-			item->tags = tags_new ();
-
-			if (title || artist || album) {
-				item->tags->filled |= TAGS_COMMENTS;
-
-				item->tags->title = title;
-				item->tags->artist = artist;
-				item->tags->album = album;
-			}
-			
-			if (time != -1) {
-				item->tags->filled |= TAGS_TIME;
-				item->tags->time = time;
-			}
-
-			item->tags->track = track;
-		}
 	}
 	
 	return item;
@@ -585,7 +625,7 @@ static char *make_event_packet (struct event *e, size_t *len)
 
 	*len = sizeof(e->type);
 
-	if (e->type == EV_PLIST_DEL) {
+	if (e->type == EV_PLIST_DEL || e->type == EV_STATUS_MSG) {
 		int str_len;
 
 		assert (e->data != NULL);
@@ -608,7 +648,7 @@ static char *make_event_packet (struct event *e, size_t *len)
 		item_packet = make_item_packet (e->data, &item_packet_len);
 		*len += item_packet_len;
 		
-		buf = xmalloc(*len);
+		buf = (char *)xmalloc(*len);
 		memcpy (buf, &e->type, sizeof(e->type));
 		memcpy (buf + sizeof(e->type), item_packet, item_packet_len);
 		free (item_packet);
@@ -649,7 +689,7 @@ enum noblock_io_status event_send_noblock (int sock, struct event_queue *q)
 			plist_free_item_fields (e->data);
 			free (e->data);
 		}
-		else if (e->type == EV_PLIST_DEL)
+		else if (e->type == EV_PLIST_DEL || e->type == EV_STATUS_MSG)
 			free (e->data);
 		else if (e->data)
 			logit ("Unhandled event data!");
