@@ -25,6 +25,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <strings.h>
 #include <signal.h>
 #include <errno.h>
 #include <stdarg.h>
@@ -180,7 +181,7 @@ static int locking_client ()
 static int client_lock (struct client *cli)
 {
 	if (cli->lock) {
-		logit ("Client tants deadlock.");
+		logit ("Client wants deadlock.");
 		return 0;
 	}
 	
@@ -212,8 +213,8 @@ static int client_unlock (struct client *cli)
 
 static void del_client (struct client *cli)
 {
-	cli->socket = -1;
 	struct event *e;
+	cli->socket = -1;
 
 	/* Free the event queue - we can't just use event_queue_free(), because
 	 * it can't free() the event's data. */
@@ -223,7 +224,7 @@ static void del_client (struct client *cli)
 			plist_free_item_fields (e->data);
 			free (e->data);
 		}
-		else if (e->type == EV_PLIST_DEL)
+		else if (e->type == EV_PLIST_DEL || e->type == EV_STATUS_MSG)
 			free (e->data);
 		event_pop (&cli->events);
 	}
@@ -244,11 +245,24 @@ static void wake_up_server ()
 	
 	debug ("Waking up the server");
 
-	if (write(wake_up_pipe[1], &w, sizeof(1)) < 0)
+	if (write(wake_up_pipe[1], &w, sizeof(w)) < 0)
 		logit ("Can't wake up the server: (write() failed) %s",
 				strerror(errno));
 }
 
+/* Thread-safe signal() version */
+static void thread_signal (const int signum, void (*func)(int))
+{
+	struct sigaction act;
+
+	act.sa_handler = func;
+	act.sa_flags = 0;
+	sigemptyset (&act.sa_mask);
+	
+	if (sigaction(signum, &act, 0) == -1)
+		fatal ("sigaction() failed: %s", strerror(errno));
+}
+
 /* Initialize the server - return fd of the listening socket or -1 on error */
 int server_init (int debug, int foreground)
 {
@@ -258,7 +272,11 @@ int server_init (int debug, int foreground)
 
 	pid = check_pid_file ();
 	if (pid && valid_pid(pid)) {
-		fprintf (stderr, "Server already running with pid %d\n", pid);
+		fprintf (stderr, "\nIt seems that the server is already running"
+				" with pid %d.\n", pid);
+		fprintf (stderr, "If it is not true, remove the pid file (%s)"
+				" and try again.\n",
+				create_file_name(PID_FILE));
 		fatal ("Exiting.");
 	}
 
@@ -293,11 +311,11 @@ int server_init (int debug, int foreground)
 	clients_init ();
 
 	server_tid = pthread_self ();
-	signal (SIGTERM, sig_exit);
-	signal (SIGINT, foreground ? sig_exit : SIG_IGN);
-	signal (SIGHUP, SIG_IGN);
-	signal (SIGQUIT, sig_exit);
-	signal (SIGPIPE, SIG_IGN);
+	thread_signal (SIGTERM, sig_exit);
+	thread_signal (SIGINT, foreground ? sig_exit : SIG_IGN);
+	thread_signal (SIGHUP, SIG_IGN);
+	thread_signal (SIGQUIT, sig_exit);
+	thread_signal (SIGPIPE, SIG_IGN);
 
 	write_pid_file ();
 
@@ -333,7 +351,7 @@ static void add_event (struct client *cli, const int event, void *data)
 	UNLOCK (cli->events_mutex);
 }
 
-static void add_event_all (const int event, void *data)
+static void add_event_all (const int event, const void *data)
 {
 	int i;
 	int added = 0;
@@ -347,7 +365,8 @@ static void add_event_all (const int event, void *data)
 					data_copy = plist_new_item ();
 					plist_item_copy (data_copy, data);
 				}
-				else if (event == EV_PLIST_DEL) {
+				else if (event == EV_PLIST_DEL
+						|| event == EV_STATUS_MSG) {
 					data_copy = xstrdup (data);
 				}
 				else
@@ -361,7 +380,8 @@ static void add_event_all (const int event, void *data)
 	if (added)
 		wake_up_server ();
 	else
-		debug ("No events were added because there are no clients.");
+		debug ("No events have been added because there are no "
+				"clients.");
 }
 
 /* Send events from the queue. Return 0 on error. */
@@ -403,6 +423,8 @@ static void server_shutdown ()
 	audio_exit ();
 	unlink (socket_name());
 	unlink (create_file_name(PID_FILE));
+	close (wake_up_pipe[0]);
+	close (wake_up_pipe[1]);
 	logit ("Server exited");
 }
 
@@ -801,7 +823,7 @@ static int gen_serial (const struct client *cli)
 
 	do {
 		serial = (seed << 8) | client_index(cli);
-		seed = (seed + 1) & 0x07; /* TODO:  it should be 0xFF */
+		seed = (seed + 1) & 0xFF;
 	} while (serial == audio_plist_get_serial());
 
 	debug ("Generated serial %d for client with fd %d", serial,
@@ -821,6 +843,51 @@ static int send_serial (struct client *cli)
 	return 1;
 }
 
+/* Send tags to the client. Return 0 on error. */
+static int req_get_tags (struct client *cli)
+{
+	struct file_tags *tags;
+	int res = 1;
+
+	debug ("Sending tags to client with fd %d...", cli->socket);
+	
+	if (!send_int(cli->socket, EV_DATA)) {
+		logit ("Error when sending EV_DATA");
+		return 0;
+	}
+	
+	tags = audio_get_curr_tags ();
+	if (!send_tags(cli->socket, tags)) {
+		logit ("Error when sending tags");
+		res = 0;
+	}
+
+	if (tags)
+		tags_free (tags);
+	
+	return res;
+}
+
+/* Handle CMD_GET_MIXER_CHANNEL_NAME. Return 0 on error. */
+int req_get_mixer_channel_name (struct client *cli)
+{
+	int status = 1;
+	char *name = audio_get_mixer_channel_name ();
+	
+	if (!send_data_str(cli, name ? name : ""))
+		status = 0;
+	free (name);
+
+	return status;
+}
+
+/* Handle CMD_TOGGLE_MIXER_CHANNEL. Return 0 on error. */
+void req_toggle_mixer_channel ()
+{
+	audio_toggle_mixer_channel ();
+	add_event_all (EV_MIXER_CHANGE, NULL);
+}
+
 /* Reveive a command from the client and execute it. */
 static void handle_command (struct client *cli)
 {
@@ -972,6 +1039,17 @@ static void handle_command (struct client *cli)
 			if (!req_plist_set_serial(cli))
 				err = 1;
 			break;
+		case CMD_GET_TAGS:
+			if (!req_get_tags(cli))
+				err = 1;
+			break;
+		case CMD_TOGGLE_MIXER_CHANNEL:
+			req_toggle_mixer_channel ();
+			break;
+		case CMD_GET_MIXER_CHANNEL_NAME:
+			if (!req_get_mixer_channel_name(cli))
+				err = 1;
+			break;
 		default:
 			logit ("Bad command (0x%x) from the client.", cmd);
 			err = 1;
@@ -1117,27 +1195,10 @@ void server_loop (int list_sock)
 	server_shutdown ();
 }
 
-/* Don't allow events to be send to quickly. Return 1 if the event can be
- * send. */
-static int event_throttle ()
-{
-	static int last_time = 0;
-
-	if (last_time != time(NULL)) {
-		last_time = time(NULL);
-		return 1;
-	}
-	
-	return 0;
-}
-
 void set_info_bitrate (const int bitrate)
 {
-	int old_bitrate = sound_info.bitrate;
-
 	sound_info.bitrate = bitrate;
-	if (old_bitrate <= 0 || bitrate <= 0 || event_throttle())
-		add_event_all (EV_BITRATE, NULL);
+	add_event_all (EV_BITRATE, NULL);
 }
 
 void set_info_channels (const int channels)
@@ -1162,3 +1223,13 @@ void ctime_change ()
 {
 	add_event_all (EV_CTIME, NULL);
 }
+
+void tags_change ()
+{
+	add_event_all (EV_TAGS, NULL);
+}
+
+void status_msg (const char *msg)
+{
+	add_event_all (EV_STATUS_MSG, msg);
+}
